@@ -20,7 +20,7 @@ come with the distribution.
 
 =cut
 
-$VERSION = '0.95_02';
+$VERSION = '0.95_03';
 use strict;
 use B;
 use Sepia::Debug;               # THIS TURNS ON DEBUGGING INFORMATION!
@@ -29,16 +29,15 @@ use Scalar::Util 'looks_like_number';
 use Text::Abbrev;
 
 use vars qw($PS1 %REPL %RK %REPL_DOC %REPL_SHORT %PRINTER
+            @REPL_RESULT
             $REPL_LEVEL $PACKAGE $WANTARRAY $PRINTER $STRICT $PRINT_PRETTY
             $ISEVAL);
 
-BEGIN {
+sub repl_strict
+{
     eval { require Lexical::Persistence; import Lexical::Persistence };
     if ($@) {
-        *repl_strict = sub {
-            print "Strict mode requires Lexical::Persistence.\n";
-            0;
-        };
+        print "Strict mode requires Lexical::Persistence.\n";
     } else {
         *repl_strict = sub {
             my $x = as_boolean(shift, $STRICT);
@@ -47,15 +46,23 @@ BEGIN {
             } elsif (!$x) {
                 undef $STRICT;
             }
-            0;
         };
+        goto &repl_strict;
     }
+}
+
+sub core_version
+{
     eval { require Module::CoreList };
     if ($@) {
-        *Sepia::core_version = sub { '???' };
+        '???';
     } else {
-        *Sepia::core_version = sub { Module::CoreList->first_release(@_) };
+        *core_version = sub { Module::CoreList->first_release(@_) };
+        goto &core_version;
     }
+}
+
+BEGIN {
     eval { use List::Util 'max' };
     if ($@) {
         *Sepia::max = sub {
@@ -66,12 +73,13 @@ BEGIN {
             $ret;
         };
     }
+}
+
+sub repl_size
+{
     eval { require Devel::Size };
     if ($@) {
-        *Sepia::repl_size = sub {
-            print "Size requires Devel::Size.\n";
-            0;
-        };
+        print "Size requires Devel::Size.\n";
     } else {
         *Sepia::repl_size = sub {
             ## XXX: C&P from repl_who:
@@ -88,14 +96,14 @@ BEGIN {
             my $fmt = '%-'.$len."s%10d\n";
             print 'Var', ' ' x ($len + 2), "Bytes\n";
             print '-' x ($len-4), ' ' x 9, '-' x 5, "\n";
-            local $SIG{WARN} = sub {};
+            local $SIG{__WARN__} = sub {};
             for (@who) {
                 my $res = eval "package $pkg; Devel::Size::total_size \\$_;";
                 next if $res == 0;
                 printf $fmt, $_, $res || 0;
             }
-            0;
         };
+        goto &repl_size;
     }
 }
 
@@ -149,26 +157,6 @@ sub _apropos_re($)
     }
 }
 
-sub _completions1
-{
-    no strict;
-    my $stash = shift;
-    my $re = shift || '';
-    $re = qr/$re/;
-    if (@_ == 0 || !defined $_[0]) {
-        map "$stash$_", grep /$re/, keys %$stash;
-    } else {
-        map {
-            _completions1("$stash$_", @_);
-        } grep /$re.*::$/, keys %$stash;
-    };
-}
-
-sub _completions
-{
-    _completions1 '::', _apropos_re($_[0]);
-}
-
 my %sigil;
 BEGIN {
     %sigil = qw(ARRAY @ SCALAR $ HASH %);
@@ -177,13 +165,19 @@ BEGIN {
 sub filter_untyped
 {
     no strict;
+    local $_ = /^::/ ? $_ : "::$_";
     defined *{$_}{CODE} || defined *{$_}{IO} || (/::$/ && defined *{$_}{HASH});
 }
 
+## XXX: Careful about autovivification here!  Specifically:
+##     defined *FOO{HASH} # => ''
+##     defined %FOO       # => ''
+##     defined *FOO{HASH} # => 1
 sub filter_typed
 {
     no strict;
     my $type = shift;
+    local $_ = /^::/ ? $_ : "::$_";
     if ($type eq 'SCALAR') {
         defined ${$_};
     } elsif ($type eq 'VARIABLE') {
@@ -193,60 +187,81 @@ sub filter_typed
     }
 }
 
-## XXX: autovivification gives us problems here sometimes.  Specifically:
-##     defined *FOO{HASH} # => ''
-##     defined %FOO       # => ''
-##     defined *FOO{HASH} # => 1
+sub maybe_icase
+{
+    my $ch = shift;
+    $ch =~ /[A-Z]/ ? $ch : '['.uc($ch).$ch.']';
+}
+
+sub all_abbrev_completions
+{
+    use vars '&_completions';
+    local *_completions = sub {
+        no strict;
+        my ($stash, @e) = @_;
+        my $ch = '[A-Za-z0-9]*';
+        my $re1 = "^".maybe_icase($e[0]).$ch.join('', map {
+            '_'.maybe_icase($_).$ch
+        } @e[1..$#e]);
+        $re1 = qr/$re1/;
+        my $re2 = maybe_icase $e[0];
+        $re2 = qr/^$re2.*::$/;
+        my @ret = grep !/::$/ && /$re1/, keys %{$stash};
+        my @pkgs = grep /$re2/, keys %{$stash};
+        (map("$stash$_", @ret),
+         @e > 1 ? map { _completions "$stash$_", @e[1..$#e] } @pkgs :
+             map { "$stash$_" } @pkgs)
+    };
+    map { s/^:://; $_ } _completions('::', split //, shift);
+}
+
+sub apropos_re
+{
+    my ($icase, $re) = @_;
+    $re =~ s/_/[^_]*_/g;
+    $icase ? qr/^$re.*$/i : qr/^$re.*$/;
+}
+
+sub all_completions
+{
+    my $icase = $_[0] !~ /[A-Z]/;
+    my @parts = split /:+/, shift, -1;
+    my $re = apropos_re $icase, pop @parts;
+    use vars '&_completions';
+    local *_completions = sub {
+        no strict;
+        my $stash = shift;
+        if (@_ == 0) {
+            map { "$stash$_" } grep /$re/, keys %{$stash};
+        } else {
+            my $re2 = $icase ? qr/^$_[0].*::$/i : qr/^$_[0].*::$/;
+            my @pkgs = grep /$re2/, keys %{$stash};
+            map { _completions "$stash$_", @_[1..$#_] } @pkgs
+        }
+    };
+    map { s/^:://; $_ } _completions('::', @parts);
+}
 
 sub completions
 {
-    no strict;
-    my ($str, $type, $infunc) = @_;
-    my @ret;
-
-    if (!$type) {
-        @ret = grep { filter_untyped } _completions $str;
-    } else {
-        @ret = grep { filter_typed $type } _completions $str;
-        if (defined $infunc && defined *{$infunc}{CODE}) {
-            my ($apre) = _apropos_re($str);
-            my $st = $sigil{$type};
-            push @ret, grep {
-                (my $tmp = $_) =~ s/^\Q$st//;
-                $tmp =~ /$apre/;
-            } lexicals($infunc);
-        }
+    my ($type, $str) = $_[0] =~ /^([\%\$\@\&]?)(.*)/;
+    my %h = qw(@ ARRAY % HASH & CODE * IO $ SCALAR);
+    my $t = $type || '';
+    $type = $h{$type} if $type;
+    my @ret = grep {
+        $type ? filter_typed $type : filter_untyped
+    } all_completions $str;
+    if (!@ret && $str !~ /:/) {
+        @ret = grep {
+            $type ? filter_typed $type : filter_untyped
+        } all_abbrev_completions $str;
     }
-
-    ## Complete "simple" sequences as abbreviations, e.g.:
-    ##   wtci -> Want_To_Complete_It, NOT
-    ##        -> WaTCh_trIpe
-    if (!@ret && $str =~ /^\w+$/) {
-        my $broad = _apropos_re join '_', split '', $str;
-        # print "abbrev completion on '$str': broad = /$broad/\n";
-        if ($type) {
-            @ret = grep { filter_typed $type } _completions1 '::', qr/$broad/;
-        } else {
-            @ret = grep { filter_untyped } _completions1 '::', qr/$broad/;
-        }
-        if (defined $infunc && defined *{$infunc}{CODE}) {
-            my $st = $sigil{$type};
-            grep {
-                (my $tmp = $_) =~ s/^\Q$st//;
-                $tmp =~ /$broad/;
-            } lexicals($infunc);
-        }
-    }
-    ## Complete packages so e.g. "new B:T" -> "new Blah::Thing"
-    ## instead of "new Blah::Thing::"
-    if (!$type) {
-        @ret = map { /(.*)::$/ ? ($1, $_) : $_ } @ret;
-    }
-    ## XXX: Control characters, $", and $1, etc. confuse Emacs, so
-    ## remove them.
+    @ret = map { s/^:://; "$t$_" } @ret;
+#     ## XXX: Control characters, $", and $1, etc. confuse Emacs, so
+#     ## remove them.
     grep {
         length > 0 && !looks_like_number $_ && !/^[^\w\d_]$/ && !/^_</ && !/^[[:cntrl:]]/
-    } map { s/^:://; $_ } @ret;
+    } @ret;
 }
 
 sub method_completions
@@ -415,14 +430,13 @@ Emacs-called function to get module information.
 
 =cut
 
-sub module_info($$);
-
-BEGIN {
+sub module_info
+{
     eval { require Module::Info; import Module::Info };
     if ($@) {
-        *module_info = sub ($$) { undef };
+        undef;
     } else {
-        *module_info = sub ($$) {
+        *module_info = sub {
             my ($m, $func) = @_;
             my $info;
             if (-f $m) {
@@ -440,6 +454,7 @@ BEGIN {
                 return $info->$func;
             }
         };
+        goto &module_info;
     }
 }
 
@@ -596,7 +611,7 @@ which can use either L<Data::Dumper>, L<YAML>, or L<Data::Dump>.
             s/;$//;
         };
         if (length $_ > ($ENV{COLUMNS} || 80)) {
-            $Data::Dumper::Indent = 2;
+            $Data::Dumper::Indent = 1;
             eval {
                 $_ = Data::Dumper::Dumper($thing);
                 s/\A\$VAR1 = //;
@@ -645,7 +660,7 @@ sub printer
         # overloaded?
         $res = $res[0];
     } elsif (!$ISEVAL && $PRINT_PRETTY && @res > 1 && !grep ref, @res) {
-        $res = columnate(sort @res);
+        $res = columnate(@res);
         print $res;
         return;
     } else {
@@ -777,7 +792,6 @@ sub repl_help
             printf "%-${left}s%s\n", $REPL_SHORT{$_}, $flow;
         }
     }
-    0;
 }
 
 sub repl_define
@@ -790,16 +804,15 @@ sub repl_define
         ($name, $doc, $body) = ($1, $2, $2);
     } else {
         print "usage: define NAME ['doc'] BODY...\n";
-        return 0;
+        return;
     }
-    my $sub = eval "sub { do { $body }; 0 }";
+    my $sub = eval "sub { do { $body } }";
     if ($@) {
         print "usage: define NAME ['doc'] BODY...\n\t$@\n";
-        return 0;
+        return;
     }
     define_shortcut $name, $sub, $doc;
     %RK = abbrev keys %REPL;
-    0;
 }
 
 sub repl_undef
@@ -816,7 +829,6 @@ sub repl_undef
     } else {
         print "$name: no such shortcut.\n";
     }
-    0;
 }
 
 sub repl_format
@@ -833,7 +845,6 @@ sub repl_format
             warn "No such format '$t' (dumper, dump, yaml, plain).\n";
         }
     }
-    0;
 }
 
 sub repl_chdir
@@ -848,13 +859,11 @@ sub repl_chdir
     } else {
         warn "Can't chdir\n";
     }
-    0;
 }
 
 sub repl_pwd
 {
     print Cwd::getcwd(), "\n";
-    0;
 }
 
 sub who
@@ -903,7 +912,6 @@ sub repl_who
         $pkg = $PACKAGE;
     }
     print columnate who($pkg || $PACKAGE, $re);
-    0;
 }
 
 sub methods
@@ -932,7 +940,6 @@ sub repl_methods
     $re ||= '.?';
     $re = qr/$re/;
     print columnate sort { $a cmp $b } grep /$re/, methods $x;
-    0;
 }
 
 sub as_boolean
@@ -945,7 +952,6 @@ sub as_boolean
 sub repl_wantarray
 {
     $WANTARRAY = as_boolean shift, $WANTARRAY;
-    0;
 }
 
 sub repl_package
@@ -959,12 +965,11 @@ sub repl_package
     } else {
         warn "Can't go to package $p -- doesn't exist!\n";
     }
-    0;
 }
 
 sub repl_quit
 {
-    1;
+    last repl;
 }
 
 sub repl_reload
@@ -982,7 +987,6 @@ sub repl_shell
 {
     my $cmd = shift;
     print `$cmd 2>& 1`;
-    return 0;
 }
 
 sub repl_eval
@@ -1035,6 +1039,15 @@ sub print_warnings
     }
 }
 
+sub repl_banner
+{
+    print <<EOS;
+I need user feedback!  Please send questions or comments to seano\@cpan.org.
+Sepia version $Sepia::VERSION.
+Type ",h" for help, or ",q" to quit.
+EOS
+}
+
 =head2 C<repl()>
 
 Execute a command interpreter on standard input and standard output.
@@ -1099,11 +1112,9 @@ sub repl
     local *__;
     local *CORE::GLOBAL::die = \&Sepia::Debug::die;
     local *CORE::GLOBAL::warn = \&Sepia::Debug::warn;
+    local @REPL_RESULT;
     Sepia::Debug::add_repl_commands;
-    print <<EOS if $REPL_LEVEL == 1;
-Sepia version $Sepia::VERSION.
-Type ",h" for help, or ",q" to quit.
-EOS
+    repl_banner if $REPL_LEVEL == 1;
     print prompt;
     my @sigs = qw(INT TERM PIPE ALRM);
     local @SIG{@sigs};
@@ -1141,10 +1152,7 @@ EOS
                     my $ret;
                     my $arg = $2;
                     chomp $arg;
-                    ($ret, @res) = $Sepia::REPL{$Sepia::RK{$short}}->($arg, wantarray);
-                    if ($ret) {
-                        return wantarray ? @res : $res[0];
-                    }
+                    $Sepia::REPL{$Sepia::RK{$short}}->($arg, wantarray);
                 } else {
                     if (grep /^$short/, keys %Sepia::REPL) {
                         print "Ambiguous shortcut '$short': ",
@@ -1195,6 +1203,7 @@ EOS
             print_warnings;
             print prompt;
         }
+    wantarray ? @REPL_RESULT : $REPL_RESULT[0]
 }
 
 sub perl_eval
